package Deja;

# $Id: Deja.pm,v 1.19 2000/11/12 03:19:58 xi Exp $

###############################################################################
#    Usenet Indexer - A nice and fast usenet indexer
#    Copyright (C) 2000  Christian Laursen <xi@borderworlds.dk>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
###############################################################################


use strict;
use URI::Escape;
use Digest::MD5 qw(md5_hex);
require Exporter;

# Common stuff for all scripts

use vars qw(@ISA @EXPORT $natchars %limits $newsserver @newsgroups @allgroups);

@ISA = qw(Exporter);
@EXPORT = qw($natchars %limits $newsserver @newsgroups @allgroups &deja_init &lc2 &index_article &msgid &getmsgid &search);

# *****************
# * Configuration *
# *****************

# Common

$natchars = "";

sub lc2 ($) {
    my ($word) = @_;
    $word =~ tr/A-Z/a-z/;
    return $word;
}

%limits = (30 => 'One month',
	   60 => 'Two months',
	   183 => 'Half a year');

# News

$newsserver = "news.worldonline.dk";
@newsgroups = ("dk.test",
	       "dk.edb.internet.webdesign.serverside");

# *******************************************************************************
# * Configuration ends here - you do not need to edit anything below this point *
# *******************************************************************************

# ******************
# * Initialization *
# ******************

@allgroups = sort @newsgroups;

mkdir ("../data", 0755) unless -d "../data";
mkdir ("../data/spool", 0755) unless -d "../data/spool";
mkdir ("../data/spool/news", 0755) unless -d "../data/spool/news";
mkdir ("../data/spool/searches", 0755) unless -d "../data/spool/searches";
mkdir ("../data/msgid", 0755) unless -d "../data/msgid";
mkdir ("../data/article", 0755) unless -d "../data/article";
mkdir ("../data/index", 0755) unless -d "../data/index";
mkdir ("../data/group", 0755) unless -d "../data/group";

# ***************
# * Common subs *
# ***************

sub deja_init {
    chdir "/home/xi/projects/deja.borderworlds.dk/docs";
}

sub index_article ($$$) {
    my ($id, $content, $groups) = @_;

    my @groups = split(/,/, $groups);
    foreach my $group (@groups) {
	open (FILE, ">>../data/group/$group") or die "Couldn't append to file: $!";
	print FILE pack("II", $id, 1);
	close FILE;
    }

    my %words = ();

    while ($content =~ m/([\w$natchars]+)/g) {
	$words{lc2($1)} += 1 if (length($1) >= 2 && length($1) <= 20);
    }

    while (my ($word, $hits) = each %words) {
	open (FILE, ">>../data/index/".uri_escape($word)) or die "Couldn't append to file: $!";
	print FILE pack("II", $id, $hits);
	close FILE;
    }
}

sub msgid {
    my ($msgid,$id) = @_;
    my $res = 0;
    
    my $file = "../data/msgid/".md5_hex($msgid);
    if (open (FILE, "<$file")) {
	while (<FILE>) {
	    if (m/^(\d+)\t(.*)$/) {
		$res = $1 if ($2 eq $msgid);
	    }
	}
	close FILE;
    }
    if (!$res && $id) {
	open (FILE, ">>$file") or die "Couldn't append to file: $!";
	print FILE "$id\t$msgid\n";
	close FILE;
    }
    return $res;
}

sub getmsgid {
    my ($id) = @_;
    my $file = sprintf ("../data/article/%08x.headers", $id);
    my $msgid;
    open (FILE, "<$file") or die "Couldn't read file: $!";
    while (<FILE>) {
	if (m/^Message-ID: (.*)$/) {
	    $msgid = $1;
	}
    }
    close FILE;
    return $msgid;
}

sub search ($$$) {
    my ($search,$groups,$maxage) = @_;
    my $resfile;
    if ($$search ne '') {
	$$search =~ s/[^\w$natchars +\-,]//g;
	$maxage =~ s/\D//g;
	my $groupcount = 0;
	for (my $i = 0; $i < @$groups; ++$i) {
	    $groups->[$i] =~ s/[^\w\.\-+]//g;
	    $groupcount++ if $groups->[$i] ne '';
	}

	my $searchstring = $maxage.'_'.join('_', @$groups)."_".uri_escape($$search);
	$resfile = "../data/spool/searches/$searchstring";
	unless (-f $resfile) {
	    my @plus = ();
	    my @minus = ();
	    my @neutral = ();
	    while ($$search =~ m/([+\-]?[\w$natchars]+)/g) {
		my $word = lc2($1);
		if ($word =~ m/^\+(.*)$/) {
		    push @plus, uri_escape ($1);
		} elsif ($word =~ m/^-(.*)$/) {
		    push @minus, uri_escape ($1);
		} else {
		    push @neutral, uri_escape ($word);
		}
	    }
	    if (@plus == 0 && @neutral > 0) {
		@plus = shift @neutral;
	    }
	    my $fromid = 0;
	    if ($maxage) {
		open (FILE, "<../data/article.limits") or die "Couldn't read file: $!";
		while (<FILE>) {
		    if (m/^$maxage: (\d+)$/) {
			$fromid = $1;
		    }
		}
		close FILE;
	    }
	    my $cmd = "../bin/search ".scalar(@plus)." ".$groupcount." ".scalar(@minus)." ".scalar(@neutral)." ".
		join (' ', @plus)." ".
		    join (' ', @$groups)." ".
			join (' ', @minus)." ".
			    join (' ', @neutral)." ".
				$searchstring." ".$fromid;
	    system ($cmd) and die ("Couldn't spawn search process $cmd: $?");
	}
    }
    return $resfile;
}

1;
